@@ -59,7 +59,9 @@ impl BasicMacro {
             ("tempto".to_owned(), BasicMacro::set_temp_target as MacroType).to_owned(),
             ("tempout".to_owned(), BasicMacro::temp as MacroType).to_owned(),
             ("tempin".to_owned(), BasicMacro::temp_include as MacroType).to_owned(),
+            ("fileout".to_owned(), BasicMacro::file_out as MacroType).to_owned(),
             ("pipe".to_owned(), BasicMacro::pipe as MacroType).to_owned(),
+            ("bind".to_owned(), BasicMacro::bind_to_local as MacroType).to_owned(),
             ("env".to_owned(), BasicMacro::get_env as MacroType).to_owned(),
             ("path".to_owned(), BasicMacro::merge_path as MacroType).to_owned(),
             ("-".to_owned(), BasicMacro::get_pipe as MacroType).to_owned(),
@@ -121,7 +123,7 @@ impl BasicMacro {
             let result = reg.replace_all(source, substitution); // This is a cow, moo~
             Ok(result.to_string())
         } else {
-            Err(RadError::InvalidArgument("Regex sub requires three arguments"))
+            Err(RadError::InvalidArgument("Regex sub requires three arguments".to_owned()))
         }
     }
 
@@ -135,7 +137,7 @@ impl BasicMacro {
             // Enable floating points length (or something similar)
             Ok(result.to_string())
         } else {
-            Err(RadError::InvalidArgument("Eval requires an argument"))
+            Err(RadError::InvalidArgument("Eval requires an argument".to_owned()))
         }
     }
 
@@ -145,7 +147,7 @@ impl BasicMacro {
         if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
             Utils::trim(&args[0])
         } else {
-            Err(RadError::InvalidArgument("Trim requires an argument"))
+            Err(RadError::InvalidArgument("Trim requires an argument".to_owned()))
         }
     }
 
@@ -159,7 +161,7 @@ impl BasicMacro {
 
             Ok(result.to_string())
         } else {
-            Err(RadError::InvalidArgument("Chomp requires an argument"))
+            Err(RadError::InvalidArgument("Chomp requires an argument".to_owned()))
         }
     }
 
@@ -172,7 +174,7 @@ impl BasicMacro {
 
             Ok(result.to_string())
         } else {
-            Err(RadError::InvalidArgument("Compress requires an argument"))
+            Err(RadError::InvalidArgument("Compress requires an argument".to_owned()))
         }
     }
 
@@ -183,10 +185,10 @@ impl BasicMacro {
             if let Ok(count) = Utils::trim(word_count)?.parse::<usize>() {
                 Ok(lipsum(count))
             } else {
-                Err(RadError::InvalidArgument("Lipsum needs a number bigger or equal to 0 (unsigned integer)"))
+                Err(RadError::InvalidArgument("Lipsum needs a number bigger or equal to 0 (unsigned integer)".to_owned()))
             }
         } else {
-            Err(RadError::InvalidArgument("Placeholder requires an argument"))
+            Err(RadError::InvalidArgument("Placeholder requires an argument".to_owned()))
         }
     }
 
@@ -195,9 +197,18 @@ impl BasicMacro {
         if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
             let raw = Utils::trim(&args[0])?;
             let file_path = std::path::Path::new(&raw);
-            Ok(processor.from_file(file_path, true)?)
+            if file_path.exists() { 
+                // This reservation is necessary because from_file method clears all locals
+                let reserved_local = processor.map.local.clone();
+                let result = processor.from_file(file_path, true)?;
+                processor.map.local = reserved_local;
+                Ok(result)
+            } else {
+                let formatted = format!("File path : \"{}\" doesn't exist", file_path.display());
+                Err(RadError::InvalidArgument(formatted))
+            }
         } else {
-            Err(RadError::InvalidArgument("Include requires an argument"))
+            Err(RadError::InvalidArgument("Include requires an argument".to_owned()))
         }
     }
 
@@ -208,7 +219,7 @@ impl BasicMacro {
             if let Ok(count) = Utils::trim(&args[0])?.parse::<usize>() {
                 repeat_count = count;
             } else {
-                return Err(RadError::InvalidArgument("Repeat needs a number bigger or equal to 0 (unsigned integer)"));
+                return Err(RadError::InvalidArgument("Repeat needs a number bigger or equal to 0 (unsigned integer)".to_owned()));
             }
             let repeat_object = &args[1];
             let mut repeated = String::new();
@@ -217,7 +228,7 @@ impl BasicMacro {
             }
             Ok(repeated)
         } else {
-            Err(RadError::InvalidArgument("Repeat requires two arguments"))
+            Err(RadError::InvalidArgument("Repeat requires two arguments".to_owned()))
         }
     }
 
@@ -245,7 +256,7 @@ impl BasicMacro {
 
             Ok(String::from_utf8(output)?)
         } else {
-            Err(RadError::InvalidArgument("Syscmd requires an argument"))
+            Err(RadError::InvalidArgument("Syscmd requires an argument".to_owned()))
         }
     }
 
@@ -264,7 +275,7 @@ impl BasicMacro {
             } else if let Ok(number) = trimmed_cond.parse::<i32>() {
                 if number != 0 { return Ok(if_state.to_owned()); }
             } else {
-                return Err(RadError::InvalidArgument("Ifelse requires either true/false or zero/nonzero integer."))
+                return Err(RadError::InvalidArgument("Ifelse requires either true/false or zero/nonzero integer.".to_owned()))
             }
             // if else statement exsits
             if args.len() >= 3 {
@@ -274,7 +285,7 @@ impl BasicMacro {
 
             Ok(String::new())
         } else {
-            Err(RadError::InvalidArgument("ifelse requires an argument"))
+            Err(RadError::InvalidArgument("ifelse requires an argument".to_owned()))
         }
     }
 
@@ -294,7 +305,7 @@ impl BasicMacro {
                 Ok("false".to_owned())
             }
         } else {
-            Err(RadError::InvalidArgument("Ifdef requires an argument"))
+            Err(RadError::InvalidArgument("Ifdef requires an argument".to_owned()))
         }
     }
 
@@ -306,7 +317,7 @@ impl BasicMacro {
             processor.map.undefine(name);
             Ok("".to_owned())
         } else {
-            Err(RadError::InvalidArgument("Undefine requires an argument"))
+            Err(RadError::InvalidArgument("Undefine requires an argument".to_owned()))
         }
     }
 
@@ -324,7 +335,7 @@ impl BasicMacro {
             }
             Ok(sums)
         } else {
-            Err(RadError::InvalidArgument("Foreach requires two argument"))
+            Err(RadError::InvalidArgument("Foreach requires two argument".to_owned()))
         }
     }
 
@@ -338,10 +349,10 @@ impl BasicMacro {
             let max: usize; 
             if let Ok(num) = Utils::trim(&args[0])?.parse::<usize>() {
                 min = num;
-            } else { return Err(RadError::InvalidArgument("Forloop's min value should be non zero positive integer")); }
+            } else { return Err(RadError::InvalidArgument("Forloop's min value should be non zero positive integer".to_owned())); }
             if let Ok(num) = Utils::trim(&args[1])?.parse::<usize>() {
                 max = num
-            } else { return Err(RadError::InvalidArgument("Forloop's min value should be non zero positive integer")); }
+            } else { return Err(RadError::InvalidArgument("Forloop's min value should be non zero positive integer".to_owned())); }
 
             for value in min..=max {
                 let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), &ITER.replace_all(expression, &value.to_string()))?;
@@ -350,7 +361,7 @@ impl BasicMacro {
 
             Ok(sums)
         } else {
-            Err(RadError::InvalidArgument("Forloop requires two argument"))
+            Err(RadError::InvalidArgument("Forloop requires two argument".to_owned()))
         }
     }
 
@@ -365,7 +376,7 @@ impl BasicMacro {
             let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), &result)?;
             Ok(result)
         } else {
-            Err(RadError::InvalidArgument("From requires two arguments"))
+            Err(RadError::InvalidArgument("From requires two arguments".to_owned()))
         }
     }
 
@@ -377,7 +388,7 @@ impl BasicMacro {
             let result = Formatter::csv_to_table(table_format, csv_content, &p.newline)?;
             Ok(result)
         } else {
-            Err(RadError::InvalidArgument("Table requires two arguments"))
+            Err(RadError::InvalidArgument("Table requires two arguments".to_owned()))
         }
     }
 
@@ -389,6 +400,16 @@ impl BasicMacro {
         Ok(String::new())
     }
 
+    /// $bind(name,value)
+    fn bind_to_local(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
+        if let Some(args) = ArgParser::args_with_len(args, 2, greedy) {
+            let name = &args[0];
+            let value = &args[1];
+            processor.map.new_local(1, name, value);
+        }
+        Ok(String::new())
+    }
+
     fn get_env(args: &str, _: bool, _: &mut Processor) -> Result<String, RadError> {
         let out = std::env::var(args)?;
         Ok(out)
@@ -402,7 +423,7 @@ impl BasicMacro {
             let out = format!("{}",&std::path::Path::new(&target).join(&added).display());
             Ok(out)
         } else {
-            Err(RadError::InvalidArgument("Path macro needs two arguments"))
+            Err(RadError::InvalidArgument("Path macro needs two arguments".to_owned()))
         }
     }
 
@@ -429,7 +450,7 @@ impl BasicMacro {
 
             Ok(String::new())
         } else {
-            Err(RadError::InvalidArgument("Rename requires two arguments"))
+            Err(RadError::InvalidArgument("Rename requires two arguments".to_owned()))
         }
     }
 
@@ -441,7 +462,7 @@ impl BasicMacro {
 
             Ok(String::new())
         } else {
-            Err(RadError::InvalidArgument("Append requires two arguments"))
+            Err(RadError::InvalidArgument("Append requires two arguments".to_owned()))
         }
     }
 
@@ -452,7 +473,7 @@ impl BasicMacro {
             let destination = &args[2].chars().collect::<Vec<char>>();
 
             if target.len() != destination.len() {
-                return Err(RadError::InvalidArgument("Tr's replacment should have same length of texts"));
+                return Err(RadError::InvalidArgument("Tr's replacment should have same length of texts".to_owned()));
             }
 
             for i in 0..target.len() {
@@ -461,7 +482,7 @@ impl BasicMacro {
 
             Ok(source)
         } else {
-            Err(RadError::InvalidArgument("Tr requires two arguments"))
+            Err(RadError::InvalidArgument("Tr requires two arguments".to_owned()))
         }
     }
 
@@ -480,7 +501,7 @@ impl BasicMacro {
                 min.replace(num);
             } else { 
                 if start.len() != 0 {
-                    return Err(RadError::InvalidArgument("Sub's min value should be non zero positive integer or empty value")); 
+                    return Err(RadError::InvalidArgument("Sub's min value should be non zero positive integer or empty value".to_owned())); 
                 }
             }
 
@@ -488,14 +509,14 @@ impl BasicMacro {
                 max.replace(num);
             } else { 
                 if end.len() != 0 {
-                    return Err(RadError::InvalidArgument("Sub's max value should be non zero positive integer or empty value")); 
+                    return Err(RadError::InvalidArgument("Sub's max value should be non zero positive integer or empty value".to_owned())); 
                 }
             }
 
             Ok(Utils::utf8_substring(source, min, max))
 
         } else {
-            Err(RadError::InvalidArgument("Sub requires some arguments"))
+            Err(RadError::InvalidArgument("Sub requires some arguments".to_owned()))
         }
     }
     fn pause(args: &str, greedy: bool, processor : &mut Processor) -> Result<String, RadError> {
@@ -511,10 +532,10 @@ impl BasicMacro {
             } 
             // Failed to evaluate
             else {
-                Err(RadError::InvalidArgument("Pause requires either true/false or zero/nonzero integer."))
+                Err(RadError::InvalidArgument("Pause requires either true/false or zero/nonzero integer.".to_owned()))
             }
         } else {
-            Err(RadError::InvalidArgument("Pause requires an argument"))
+            Err(RadError::InvalidArgument("Pause requires an argument".to_owned()))
         }
     }
 
@@ -522,21 +543,60 @@ impl BasicMacro {
         if let Some(args) = ArgParser::args_with_len(args, 2, greedy) {
             let truncate = &args[0];
             let content = &args[1];
-            if let Ok(value) = Utils::is_arg_true(truncate) {
+            if let Ok(truncate) = Utils::is_arg_true(truncate) {
                 let file = temp_dir().join(&p.temp_target);
-                let mut temp_file = OpenOptions::new()
-                    .create(true)
-                    .write(true)
-                    .truncate(value)
-                    .open(file)
-                    .unwrap();
+                let mut temp_file; 
+                if truncate {
+                    temp_file = OpenOptions::new()
+                        .create(true)
+                        .write(true)
+                        .truncate(true)
+                        .open(file)
+                        .unwrap();
+                } else {
+                    temp_file = OpenOptions::new()
+                        .append(true)
+                        .open(file)
+                        .unwrap();
+                }
                 temp_file.write_all(content.as_bytes())?;
                 Ok(String::new())
             } else {
-                Err(RadError::InvalidArgument("Temp requires either true/false or zero/nonzero integer."))
+                Err(RadError::InvalidArgument("Temp requires either true/false or zero/nonzero integer.".to_owned()))
             }
         } else {
-            Err(RadError::InvalidArgument("Temp requires an argument"))
+            Err(RadError::InvalidArgument("Temp requires an argument".to_owned()))
+        }
+    }
+
+    fn file_out(args: &str, greedy: bool, _: &mut Processor) -> Result<String, RadError> {
+        if let Some(args) = ArgParser::args_with_len(args, 3, greedy) {
+            let truncate = &args[0];
+            let file_name = &args[1];
+            let content = &args[2];
+            if let Ok(truncate) = Utils::is_arg_true(truncate) {
+                let file = std::env::current_dir()?.join(file_name);
+                let mut target_file; 
+                if truncate {
+                    target_file = OpenOptions::new()
+                        .create(true)
+                        .write(true)
+                        .truncate(true)
+                        .open(file)
+                        .unwrap();
+                    } else {
+                        target_file = OpenOptions::new()
+                            .append(true)
+                            .open(file)
+                            .unwrap();
+                }
+                target_file.write_all(content.as_bytes())?;
+                Ok(String::new())
+            } else {
+                Err(RadError::InvalidArgument("Temp requires either true/false or zero/nonzero integer.".to_owned()))
+            }
+        } else {
+            Err(RadError::InvalidArgument("Temp requires an argument".to_owned()))
         }
     }
 
@@ -550,7 +610,7 @@ impl BasicMacro {
             processor.temp_target = PathBuf::from(std::env::temp_dir()).join(&args[0]);
             Ok(String::new())
         } else {
-            Err(RadError::InvalidArgument("Temp requires an argument"))
+            Err(RadError::InvalidArgument("Temp requires an argument".to_owned()))
         }
     }
 }
