@@ -18,6 +18,7 @@ pub struct MacroFragment {
     pub greedy: bool,
     pub preceding: bool,
     pub yield_literal : bool,
+    pub trimmed : bool,
 }
 
 impl MacroFragment {
@@ -30,6 +31,7 @@ impl MacroFragment {
             greedy: false,
             preceding: false,
             yield_literal : false,
+            trimmed: false,
         }
     }
 
@@ -40,6 +42,7 @@ impl MacroFragment {
         self.pipe = false; 
         self.greedy = false; 
         self.yield_literal = false;
+        self.trimmed= false; 
     }
 
     pub fn is_empty(&self) -> bool {
@@ -282,6 +285,7 @@ impl Processor {
                                 '|' => frag.pipe = true,
                                 '+' => frag.greedy = true,
                                 '*' => frag.yield_literal = true,
+                                '^' => frag.trimmed = true,
                                 _ => frag.name.push(ch) 
                             }
                         },
@@ -307,7 +311,23 @@ impl Processor {
                     } 
                     // Invoke macro
                     else {
-                        if let Some(mut content) = self.evaluate(level, caller, &frag.name, &frag.args, frag.greedy || self.always_greedy)? {
+                        // Try to evaluate
+                        let evaluation_result = self.evaluate(level, caller, &frag.name, &frag.args, frag.greedy || self.always_greedy);
+
+                        // If panicked, this means unrecoverable error occured.
+                        if let Err(error) = evaluation_result {
+                            // this is equlvalent to conceptual if let not pattern
+                            if let RadError::Panic = error{
+                                // Do nothing
+                                ();
+                            } else {
+                                self.log_error(&format!("{}", error))?;
+                            }
+                            return Err(RadError::Panic);
+                        }
+                        // else it is ok to proceed.
+                        // thus it is safe to unwrap it
+                        if let Some(mut content) = evaluation_result.unwrap() {
                             if frag.pipe {
                                 self.pipe_value = content;
                                 lexor.escape_nl = true;
@@ -317,6 +337,9 @@ impl Processor {
                             else if content.len() == 0 {
                                 lexor.escape_nl = true;
                             } else {
+                                if frag.trimmed {
+                                    content = Utils::trim(&content)?;
+                                }
                                 if frag.yield_literal {
                                     content = format!("\\*{}*\\", content);
                                 }
@@ -354,7 +377,6 @@ impl Processor {
 
     fn add_define(&mut self, frag: &mut MacroFragment, remainder: &mut String) -> Result<(), RadError> {
         if let Some((name,args,body)) = self.define_parse.parse_define(&frag.args) {
-            println!("Register : {}\n{}\n{}", name, args, body);
             self.map.register(&name, &args, &body)?;
         } else {
             self.log_error(&format!(
@@ -370,7 +392,7 @@ impl Processor {
 
     // Evaluate can be nested deeply
     // Disable caller for temporary
-    fn evaluate(&mut self,level: usize, _caller: &str, name: &str, args: &String, greedy: bool) -> Result<Option<String>, RadError> {
+    fn evaluate(&mut self,level: usize, _caller: &str, name: &str, args: &str, greedy: bool) -> Result<Option<String>, RadError> {
         let level = level + 1;
         // This parses and processes arguments
         // and macro should be evaluated after
@@ -520,13 +542,16 @@ impl DefineParser {
                         bind = true;
                         continue;
                     } 
-                    // This means pattern like this
-                    // $define( name ) -> name is registered
-                    // $define( na me ) -> only "na" is registered
-                    else if Utils::is_blank_char(ch) && self.name.len() != 0 {
-                        self.name.push_str(&self.container);
-                        self.container.clear();
-                        self.arg_cursor = DefineCursor::Args;
+                    else if Utils::is_blank_char(ch) {
+                        // This means pattern like this
+                        // $define( name ) -> name is registered
+                        // $define( na me ) -> na is ignored and take me instead
+                        if self.name.len() != 0 {
+                            self.container.clear();
+                        } else {
+                            // Ignore
+                            continue;
+                        }
                     } 
                     // Comma go to args
                     else if ch == ',' {
